package com.xkrs.controller;

import com.xkrs.common.encapsulation.PromptMessageEnum;
import com.xkrs.dao.StreetDao;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import static com.xkrs.common.encapsulation.OutputEncapsulation.outputEncapsulationObject;

@RestController
@RequestMapping(value = "/api/adm")
public class StreettController {

    @Resource
    private StreetDao streetDao;

    /**
     * 获取省列表
     */
    @GetMapping("/getProvinceList")
    public String getProvinceList() {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        List<Map<String, String>> provinceList = streetDao.selectProvinceList();
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, provinceList, locale);
    }

    /**
     * 根据省编号获取市列表
     *
     * @param provinceCode 省编号
     * @return 市列表
     */
    @GetMapping("/getCityList")
    public String getCityList(@RequestParam("provinceCode") String provinceCode) {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        List<Map<String, String>> cityList = streetDao.selectCityList(provinceCode);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, cityList, locale);
    }

    /**
     * 根据市编号获取区县列表
     *
     * @param cityCode 市编号
     * @return 区县列表
     */
    @GetMapping("/getCountyList")
    public String getCountyList(@RequestParam("cityCode") String cityCode) {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        List<Map<String, String>> countyList = streetDao.selectCountyList(cityCode);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, countyList, locale);
    }

    /**
     * 根据区县编号获取街道列表
     *
     * @param countyCode 区县编号
     * @return 街道列表
     */
    @GetMapping("/getStreetList")
    public String getStreetList(@RequestParam("countyCode") String countyCode) {
        // 获取区域信息
        Locale locale = LocaleContextHolder.getLocale();
        List<Map<String, String>> streetList = streetDao.selectStreetList(countyCode);
        return outputEncapsulationObject(PromptMessageEnum.SUCCESS, streetList, locale);
    }

}
